# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2025 Florian Obersteiner <f.obersteiner@kit.edu>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import logging as log
from pathlib import Path

import dash_bootstrap_components as dbc
import pandas as pd
from dash import Dash

from src.callbacks import (
    register_info_updaters,
    register_map_plot,
    register_timeseries_plot,
)
from src.config import DotDict, get_latest_semantic_version_tag
from src.data import load_from_disk, load_from_thredds
from src.layout import create_layout
from src.thredds import get_datasets

appconfig: DotDict = DotDict.from_toml_file("./config/env.toml")
appconfig.VERSION = get_latest_semantic_version_tag() or appconfig.VERSION

varconfig: pd.DataFrame = pd.read_csv("./config/parms_units.csv", sep=";")
varconfig = varconfig.set_index("VNAME")

airportsconfig: pd.DataFrame = pd.read_csv("./config/CARIBIC2_Airports.csv", sep=";")
airportsconfig = airportsconfig.set_index("IATA")

# using separate variable declaration here to have a type annotation:
data: pd.DataFrame = pd.DataFrame()
ap_info: dict[int, str] = {}
data, ap_info = (
    load_from_thredds(get_datasets(appconfig), appconfig, airportsconfig, True)  # type: ignore
    if appconfig.USE_THREDDS
    else load_from_disk(Path("./testdata").glob("MS_*.nc"), appconfig, airportsconfig, True)  # type: ignore
)
log.debug(f"Data loaded. Used THREDDS: {appconfig.USE_THREDDS}")

# fmt: off
app = Dash(
    appconfig.TITLE,
    requests_pathname_prefix=appconfig.REQUESTS_PATH_PREFIX,
    external_stylesheets=[dbc.themes.BOOTSTRAP, dbc.icons.FONT_AWESOME],
)
# fmt: on

app.title = appconfig.TITLE
app.css.config.serve_locally = True
app.scripts.config.serve_locally = True

app.layout = create_layout(data, appconfig)
server = app.server  # for deployment via gunicorn / WSGI server


register_info_updaters(app, varconfig, ap_info)
register_map_plot(app, data, varconfig)
register_timeseries_plot(app, data, varconfig)


if __name__ == "__main__":
    app.run(debug=appconfig.DEBUG)  # type: ignore
