import pandas as pd
import plotly.graph_objects as go
from dash import Dash
from dash.dependencies import Input, Output
from plotly.subplots import make_subplots


def register_info_updaters(app: Dash, var_info: pd.DataFrame, ap_info: dict[int, str]):
    """
    Register callbacks for updating flight- and variable information.

    html fields / ids to update:
        - flight-dep-dest
        - var-info
        - second-var-info
    """

    @app.callback(Output("flight-dep-dest", "children"), Input("flight-dropdown", "value"))
    def set_flight_dep_dest(selected_flight: int) -> str:
        return ap_info.get(selected_flight, "?!")

    @app.callback(Output("var-info", "children"), Input("variable-dropdown", "value"))
    def set_var_info(selected_variable: str) -> str:
        return var_info.loc[selected_variable, :].Long_Name

    @app.callback(Output("second-var-info", "children"), Input("second-variable-dropdown", "value"))
    def set_second_var_info(secondary_variable: str) -> str:
        return (
            var_info.loc[secondary_variable, :].Long_Name
            if secondary_variable in var_info.index
            else ""
        )


def register_map_plot(app: Dash, timeseries_data: pd.DataFrame, var_info: pd.DataFrame):
    """
    Register callbacks for updating map plot.

    html field / id to update:
        - fig-plot
    """

    @app.callback(
        [Output("fig-map", "figure")],
        [
            Input("flight-dropdown", "value"),
            Input("variable-dropdown", "value"),
            Input("trajectories-checkbox", "value"),
        ],
    )
    def update_map_plot(selected_flight: int, primary_variable: str, show_trajectories: bool):
        filtered_df = timeseries_data[timeseries_data["flight_number"] == selected_flight]
        valid_indices = filtered_df[primary_variable].notna()
        unit_primary = var_info.loc[primary_variable, :].Unit

        if show_trajectories:
            print("ON")
            # try to find the trajectory file for selected flight
            # show pop up if no trajectory file found ?
            # make plots if trajectory file found

        fig_map = go.Figure(
            go.Scattermap(
                lon=filtered_df.loc[valid_indices, "lon"],
                lat=filtered_df.loc[valid_indices, "lat"],
                mode="markers",
                marker={
                    "color": filtered_df.loc[valid_indices, primary_variable],
                    "colorscale": "Rainbow",
                    "colorbar": {
                        "x": 0.995,
                        "title": {"text": f"{primary_variable} {unit_primary}", "side": "right"},
                    },
                    "showscale": True,
                },
                connectgaps=False,
                hoverinfo="text",
                hovertext=[f"{v:.1f}" for v in filtered_df[primary_variable]],
                name="Flight Path",
            )
        )
        fig_map.add_trace(  # marker to indicate departure airport
            go.Scattermap(
                lat=[filtered_df.lat.iloc[0]],
                lon=[filtered_df.lon.iloc[0]],
                mode="markers",
                marker={"size": 15, "allowoverlap": True, "symbol": "airport", "angle": 55},
                marker_symbol="airport",
                text=["departure airport"],
                hoverinfo="text",
                name="departure",
            )
        )
        fig_map.add_trace(  # marker to indicate destination airport
            go.Scattermap(
                lat=[filtered_df.lat.iloc[-1]],
                lon=[filtered_df.lon.iloc[-1]],
                mode="markers",
                marker={"size": 15, "allowoverlap": True, "symbol": "airport", "angle": 235},
                text=["destination airport"],
                hoverinfo="text",
                name="destination",
            )
        )

        fig_map.update_layout(
            map={
                "style": "outdoors",
                "zoom": 1.7,
                "center": {"lat": filtered_df.lat.mean(), "lon": filtered_df.lon.mean()},
            },
            margin={"l": 0, "r": 10, "t": 0, "b": 0},
            showlegend=False,
        )

        return [fig_map]


def register_timeseries_plot(app: Dash, timeseries_data: pd.DataFrame, var_info: pd.DataFrame):
    """
    Register a callback for timeseries plot.

    html field / id to update:
        - fig_ts
    """

    @app.callback(
        [Output("fig-ts", "figure")],
        [
            Input("flight-dropdown", "value"),
            Input("variable-dropdown", "value"),
            Input("second-variable-dropdown", "value"),
        ],
    )
    def update_ts_plot(selected_flight: int, primary_variable: str, secondary_variable: str):
        filtered_df = timeseries_data[timeseries_data["flight_number"] == selected_flight]
        valid_indices = filtered_df[primary_variable].notna()
        unit_primary = var_info.loc[primary_variable, :].Unit
        unit_secondary = (
            var_info.loc[secondary_variable, :].Unit
            if secondary_variable in var_info.index
            else "-"
        )

        fig_ts = make_subplots(specs=[[{"secondary_y": True}]])
        fig_ts.add_trace(
            go.Scatter(
                x=filtered_df.index[valid_indices & ~filtered_df["BB_flag"]],
                y=filtered_df.loc[valid_indices & ~filtered_df["BB_flag"], primary_variable],
                mode="markers",
                marker={
                    "size": 3,
                    "color": "blue",
                    "showscale": False,
                },
                showlegend=False,
            )
        )
        fig_ts.add_trace(
            go.Scatter(
                x=filtered_df.index[valid_indices & filtered_df["BB_flag"]],
                y=filtered_df.loc[valid_indices & filtered_df["BB_flag"], primary_variable],
                mode="markers",
                marker={
                    "size": 3,
                    "color": "red",
                    "showscale": False,
                },
                name="flagged 'biomass burning'",
            )
        )
        if secondary_variable != "None":
            fig_ts.add_trace(
                go.Scatter(
                    x=filtered_df.index,
                    y=filtered_df[secondary_variable],
                    mode="markers",
                    marker={
                        "size": 2,
                        "color": "black",
                        "showscale": False,
                    },
                    name=secondary_variable,
                ),
                secondary_y=True,
            )

        fig_ts.update_layout(
            paper_bgcolor="#e8e8e8",
            margin={"l": 45, "r": 10, "t": 20, "b": 50},
            legend={
                "x": 0.01,
                "y": 0.99,
                "xanchor": "left",
                "yanchor": "top",
                "bgcolor": "rgba(255, 255, 255, 0.7)",
                "bordercolor": "rgba(0, 0, 0, 0.2)",
                "borderwidth": 1,
            },
        )

        fig_ts.update_xaxes(title="UTC")
        fig_ts.update_yaxes(title=f"{primary_variable} {unit_primary}")
        fig_ts.update_yaxes(title=f"{secondary_variable} {unit_secondary}", secondary_y=True)

        return [fig_ts]
